<?php

/**
 * Implements hook_node_info();
 */
function tripal_analysis_kegg_node_info() {
  $nodes = array();
  $nodes['chado_analysis_kegg'] = array(
    'name' => t('Analysis: KEGG'),
    'base' => 'chado_analysis_kegg',
    'description' => t('Results from a KEGG/KAAS analysis'),
    'has_title' => FALSE,
    'title_label' => t('Analysis: KEGG'),
    'has_body' => FALSE,
    'body_label' => t('KEGG Analysis Description'),
    'locked' => TRUE,
    'chado_node_api' => array(
      'base_table' => 'analysis',
      'hook_prefix' => 'chado_analysis_kegg',
      'linking_table' => 'chado_analysis',
      'record_type_title' => array(
        'singular' => t('InterPro Analysis'),
        'plural' => t('InterPro Analyses')
      ),
      'sync_filters' => array(
        'type_id' => FALSE,
        'organism_id' => FALSE,
        'checkboxes' => array('name'),
      ),
    )
  );
  return $nodes;
}
/**
 * Implements hook_chado_node_sync_select_query().
 */
function chado_analysis_kegg_chado_node_sync_select_query($query) {
  $query['joins'][] = 'INNER JOIN {analysisprop} PROP ON PROP.analysis_id = ANALYSIS.analysis_id';
  $query['joins'][] = 'INNER JOIN {cvterm} CVTP ON CVTP.cvterm_id = PROP.type_id';
  $query['where_clauses']['analysis_type'][] = 'CVTP.name = :type_name';
  $query['where_clauses']['property_value'][] = 'PROP.value = :prop_value';
  $query['where_args']['analysis_type'][':type_name'] = 'Analysis Type';
  $query['where_args']['property_value'][':prop_value'] = 'kegg_analysis';

  return $query;
}
/*******************************************************************************
 *  The following function proves access control for users trying to
 *  perform actions on data managed by this module
 */
function chado_analysis_kegg_access($op, $node, $account) {
  if ($op == 'create') {
    if (!user_access('create chado_analysis_kegg content', $account)) {
      return FALSE;
    }
  }
  if ($op == 'update') {
    if (!user_access('edit chado_analysis_kegg content', $account)) {
      return FALSE;
    }
  }
  if ($op == 'delete') {
    if (!user_access('delete chado_analysis_kegg content', $account)) {
      return FALSE;
    }
  }
  if ($op == 'view') {
    if (!user_access('access chado_analysis_kegg content', $account)) {
      return FALSE;
    }
  }
  return NULL;
}

/*******************************************************************************
 *  Provide a KEGG Analysis form
*/
function chado_analysis_kegg_form($node, $form_state) {

  // add in the default fields
  $form = chado_analysis_form($node, $form_state);

  // set the defaults
  $hierfile = '';
  $query_re = '';
  $query_type = '';
  $query_uniquename = '';
  $organism_id = '';

  if (property_exists($node, 'analysis')) {
    $kegg = $node->analysis->tripal_analysis_kegg;
    $query_re = $kegg->query_re;
    $query_type = $kegg->query_type;
    $query_uniquename = $kegg->query_uniquename;
    $hierfile = $kegg->hierfile;
    $organism_id = $kegg->organism_id;
  }

  $moreSettings['kegg'] = 'KEGG Analysis Settings';
  $form['kegg'] = array(
    '#title' => t('KEGG Settings'),
    '#type' => 'fieldset',
    '#description' => t('Specific Settings for KEGG Analysis.'),
    '#collapsible' => TRUE,
    '#attributes' => array('id' => 'kegg-extra-settings'),
    '#group' => 'chado_node_api'
  );
  $form['kegg']['hierfile'] = array(
    '#title' => t('KAAS hier.tar.gz Output File'),
    '#type' => 'textfield',
    '#description' => t('The full path to the hier.tar.gz file generated by KAAS.
        Alternatively, you can input the full path to the directory
        that contains decompressed kegg files.'),
    '#default_value' => $hierfile,
  );
  $form['kegg']['query_re'] = array(
      '#title' => t('Query Name RE'),
      '#type' => 'textfield',
      '#description' => t('Enter the regular expression that will extract the '.
          'feature name from the results line in the KEGG heir results. This will be '.
          'the same as the definition line in the query FASTA file used for the analysis.  This option is '.
          'is only required when the query does not identically match a feature '.
          'in the database.'),
      '#default_value' => $query_re,
  );
  $cv = tripal_get_cv(array('name' => 'sequence'));
  $cv_id = $cv->cv_id;
  $form['kegg']['query_type'] = array(
    '#title' => t('Query Type'),
    '#type' => 'textfield',
    '#description' => t('Please enter the Sequence Ontology term that describes '.
        'the query sequences used for KEGG.  This is only necessary if two '.
        'or more sequences have the same name.'),
    '#default_value' => $query_type,
    '#autocomplete_path' => "admin/tripal/chado/tripal_cv/cvterm/auto_name/$cv_id",
  );

  $sql = "
    SELECT organism_id, genus, species
    FROM {organism}
    ORDER BY genus,species
  ";
  $orgs = chado_query($sql);
  $organisms = array();
  $organisms[] = '';
  while($organism = $orgs->fetchObject()) {
    $organisms[$organism->organism_id] = $organism->genus . " " . $organism->species;
  }
  $form['kegg']['organism_id'] = array(
    '#type' => 'select',
    '#title' => t('Organism'),
    '#description' => t('Select the organism for these KEGG results. The
        feature names in the KAAS .hier files must exist for the specified organism.'),
    '#options' => $organisms,
    '#default_value' => $organism_id
  );

  $form['kegg']['query_uniquename'] = array(
      '#title' => t('Use Unique Name'),
      '#type' => 'checkbox',
      '#description' => t('Select this checboxk if the feature name in the KEGG heir file '.
          'matches the uniquename in the database.  By default, the feature will '.
          'be mapped to the "name" of the feature.'),
      '#default_value' => $query_uniquename,
  );
  $form['kegg']['keggjob'] = array(
      '#type' => 'checkbox',
      '#title' => t('Submit a job to parse the kegg output into Chado'),
      '#description' => t('Note: features used in the KAAS analysis must '.
          'exist in chado before parsing the file. Otherwise, KEGG '.
          'results that cannot be linked to a feature will be '.
          'discarded.'),
  );

  return $form;
}
/**
 *
 */
function chado_analysis_kegg_validate($node, &$form, &$form_state) {
  // use the analysis parent to validate the node
  tripal_analysis_validate($node, $form, $form_state);

  // trim character fields
  $node->hierfile   = trim($node->hierfile);
  $node->query_type = trim($node->query_type);

  if($node->hierfile and !is_readable($node->hierfile)) {
    form_set_error('hierfile', "Can not read KAAS hier.tar.gz output file: '$node->hierfile'. Please check file permissions or typos in the file path.");
  }
}
/**
 *
 */
function chado_analysis_kegg_insert($node) {

  // insert the analysis. If the analysis already exist then this
  // call will link it to a new Drupa node.
  chado_analysis_insert($node);

  // trim character fields
  $node->hierfile   = trim($node->hierfile);
  $node->query_type = trim($node->query_type);

  $record = array('table'=> 'analysis', 'id' => $node->analysis_id);

  // Add the analysis type.
  chado_insert_property($record, array('type_name' =>'Analysis Type', 'cv_name' => 'analysis_property', 'value' => 'kegg_analysis'));

  // now add in the remaining settings as a single property but separated by bars
  chado_insert_property($record, array('type_name' => 'analysis_kegg_settings', 'cv_name' => 'tripal', 'value' => trim($node->hierfile)));
  chado_insert_property($record, array('type_name' => 'analysis_kegg_query_re', 'cv_name' => 'tripal', 'value' => trim($node->query_re)));
  chado_insert_property($record, array('type_name' => 'analysis_kegg_query_type', 'cv_name' => 'tripal', 'value' => trim($node->query_type)));
  chado_insert_property($record, array('type_name' => 'analysis_kegg_query_uniquename', 'cv_name' => 'tripal', 'value' => trim($node->query_uniquename)));
  chado_insert_property($record, array('type_name' => 'analysis_kegg_organism_id', 'cv_name' => 'tripal', 'value' => $node->organism_id));

  // Add a job if the user wants to parse the html output
  chado_analysis_kegg_submit_job($node);
}

/**
 * Update KEGG analysis
 */
function chado_analysis_kegg_update($node) {

  // insert the analysis
  chado_analysis_update($node);

  // trim character fields
  $node->hierfile   = trim($node->hierfile);
  $node->query_type = trim($node->query_type);

  $record = array('table'=> 'analysis', 'id' => $node->analysis_id);
  $options = array('insert_if_missing' => TRUE);

  // Update the analysis type.
  chado_update_property($record, array('type_name' =>'Analysis Type', 'cv_name' => 'analysis_property', 'value' => 'kegg_analysis'), $options);

  // now add in the remaining settings as a single property but separated by bars
  chado_update_property($record, array('type_name' => 'analysis_kegg_settings', 'cv_name' => 'tripal', 'value' => trim($node->hierfile)), $options);
  chado_update_property($record, array('type_name' => 'analysis_kegg_query_re', 'cv_name' => 'tripal', 'value' => trim($node->query_re)), $options);
  chado_update_property($record, array('type_name' => 'analysis_kegg_query_type', 'cv_name' => 'tripal', 'value' => trim($node->query_type)), $options);
  chado_update_property($record, array('type_name' => 'analysis_kegg_query_uniquename', 'cv_name' => 'tripal', 'value' => trim($node->query_uniquename)), $options);
  chado_update_property($record, array('type_name' => 'analysis_kegg_organism_id', 'cv_name' => 'tripal', 'value' => $node->organism_id), $options);

  // Add a job if the user wants to parse the output
  chado_analysis_kegg_submit_job($node);
}

/**
 * Delete KEGG anlysis
 */
function chado_analysis_kegg_delete($node) {
  chado_analysis_delete($node);
}

/**
 *
 */
function chado_analysis_kegg_submit_job($node) {
  global $user;
  global $base_url;

  if ($node->keggjob) {
    $job_args[0] = $node->analysis_id;
    $job_args[1] = $node->hierfile;
    $job_args[2] = $base_url;
    $job_args[3] = $node->query_re;
    $job_args[4] = $node->query_type;
    $job_args[5] = $node->query_uniquename;
    $job_args[6] = $node->organism_id;

    $fname = preg_replace("/.*\/(.*)/", "$1", $node->hierfile);
    tripal_add_job("Parse KAAS output: $fname", 'tripal_analysis_kegg',
      'tripal_analysis_kegg_parseHierFile', $job_args, $user->uid);
  }
}



/**
 *  When a node is requested by the user this function is called to allow us
 *  to add auxiliary data to the node object.
 */
function chado_analysis_kegg_load($nodes) {

  // load the default set of analysis fields
  chado_analysis_load ($nodes);

  foreach ($nodes as $nid => $node) {
    // create some variables for easier lookup
    $analysis = $node->analysis;
    $analysis_id = $analysis->analysis_id;
    $record = array('table'=> 'analysis', 'id' => $analysis->analysis_id);

    // get the heirfile name
    $hierfile         = chado_get_property($record, array('type_name' => 'analysis_kegg_settings', 'cv_name' => 'tripal'));
    $query_re         = chado_get_property($record, array('type_name' => 'analysis_kegg_query_re', 'cv_name' => 'tripal'));
    $query_type       = chado_get_property($record, array('type_name' => 'analysis_kegg_query_type', 'cv_name' => 'tripal'));
    $query_uniquename = chado_get_property($record, array('type_name' => 'analysis_kegg_query_uniquename', 'cv_name' => 'tripal'));
    $organism_id      = chado_get_property($record, array('type_name' => 'analysis_kegg_organism_id', 'cv_name' => 'tripal'));

    $analysis->tripal_analysis_kegg = new stdClass();
    $analysis->tripal_analysis_kegg->hierfile = is_object($hierfile)   ? $hierfile->value : '';
    $analysis->tripal_analysis_kegg->query_re = is_object($query_re)  ? $query_re->value: '';
    $analysis->tripal_analysis_kegg->query_type = is_object($query_type)  ? $query_type->value : '';
    $analysis->tripal_analysis_kegg->query_uniquename = is_object($query_uniquename)  ? $query_uniquename->value : '';
    $analysis->tripal_analysis_kegg->organism_id = is_object($organism_id) ? $organism_id->value : '';

    // Now get the title
    $node->title = chado_get_node_title($node);

    $nodes[$nid]->analysis = $analysis;
  }
}

/**
 *
 */
function chado_analysis_kegg_view($node, $teaser = FALSE, $page = FALSE) {
  // use drupal's default node view:
  if (!$teaser) {
    $node = node_prepare($node, $teaser);
    // When previewing a node submitting form, it shows 'Array' instead of
    // correct date format. We need to format the date here
    $time = $node->timeexecuted;
    if (is_array($time)) {
      $month = $time['month'];
      $day = $time['day'];
      $year = $time['year'];
      $timestamp = $year . '-' . $month . '-' . $day;
      $node->timeexecuted = $timestamp;
    }
  }
  return $node;
}


/**
 * HOOK: Implementation of hook_nodeapi()
 * Display library information for associated features or organisms
 * This function also provides contents for indexing
 */
function tripal_analysis_kegg_node_view($node, $view_mode, $langcode) {
  switch ($node->type) {
    case 'chado_analysis_kegg':
      if ($view_mode == 'full') {
        $node->content['tripal_analysis_kegg_base'] = array(
          '#markup' => theme('tripal_analysis_kegg_base', array('node' => $node)),
          '#tripal_toc_id'    => 'base',
          '#tripal_toc_title' => 'Overview',
          '#weight' => -100,
        );
        $node->content['tripal_analysis_kegg_report'] = array(
          '#markup' => theme('tripal_analysis_kegg_report', array('node' => $node)),
          '#tripal_toc_id'    => 'report',
          '#tripal_toc_title' => 'KEGG Report',
        );
      }
      if ($view_mode == 'teaser') {
        $node->content['tripal_analysis_kegg_teaser'] = array(
          '#markup' => theme('tripal_analysis_kegg_teaser', array('node' => $node)),
        );
      }
    break;
    case 'chado_organism':
      // Show KEGG report on the organism page
      if ($view_mode == 'full') {
        $node->content ['tripal_organism_kegg_summary'] = array (
          '#markup' => theme('tripal_organism_kegg_summary', array('node' => $node)),
          '#tripal_toc_id'    => 'kegg_summary',
          '#tripal_toc_title' => 'KEGG Report'
        );
      }
      break;
  }
}

/**
 * Implements [content_type]_chado_node_default_title_format().
 *
 * Defines a default title format for the Chado Node API to set the titles on
 * Chado Analysis nodes based on chado fields.
 */
function chado_analysis_kegg_chado_node_default_title_format() {
  return '[analysis.name]';
}

/**
 * Implements hook_node_insert().
 * Acts on all content types.
 *
 * @ingroup tripal_analysis_kegg
 */
function tripal_analysis_kegg_node_insert($node) {

  switch ($node->type) {
    case 'chado_analysis_kegg':

      // build the analysis variable
      $analysis_id = chado_get_id_from_nid('analysis', $node->nid);
      $values = array('analysis_id' => $analysis_id);
      $analysis = chado_generate_var('analysis', $values);
      $node->analysis = $analysis;

      // Now get the title
      $node->title = chado_get_node_title($node);

      // Now use the API to set the path.
      chado_set_node_url($node);

      break;
  }
}

/**
 * Implements hook_node_update().
 * Acts on all content types.
 *
 * @ingroup tripal_analysis_kegg
 */
function tripal_analysis_kegg_node_update($node) {

  switch ($node->type) {
    case 'chado_analysis_kegg':

      // build the analysis variable
      $analysis_id = chado_get_id_from_nid('analysis', $node->nid);
      $values = array('analysis_id' => $analysis_id);
      $analysis = chado_generate_var('analysis', $values);
      $node->analysis = $analysis;

      // Now get the title
      $node->title = chado_get_node_title($node);

      break;
  }
}